/*
* Licensed to the Apache Software Foundation (ASF) under one or more
*  contributor license agreements.  The ASF licenses this file to You
* under the Apache License, Version 2.0 (the "License"); you may not
* use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.  For additional information regarding
* copyright in this work, please see the NOTICE file in the top level
* directory of this distribution.
*/
/*
 * Filename: AcronymsPlugin.java
 * 
 * Created on 22-Jun-04
 */
package org.apache.roller.presentation.velocity.plugins.acronyms;

import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.velocity.context.Context;
import org.apache.roller.RollerException;
import org.apache.roller.model.RollerFactory;
import org.apache.roller.model.UserManager;
import org.apache.roller.pojos.WeblogTemplate;
import org.apache.roller.pojos.WeblogEntryData;
import org.apache.roller.pojos.WebsiteData;
import org.apache.roller.model.PagePlugin;

import java.util.Iterator;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Adds full text to pre-defined acronyms.
 * 
 * Example: HTML would become &lt;acronym title="Hyper Text Markup Language"&gt;HTML&lt;/acronym&gt; 
 * 
 * @author <a href="mailto:molen@mail.com">Jaap van der Molen</a>
 * @version $Revision: 1.3 $
 */
public class AcronymsPlugin implements PagePlugin
{
	/**
	 * Logger
	 */
	private static final Log mLogger = LogFactory.getLog(AcronymsPlugin.class);

	/**
	 * Name of this Plugin.
	 */
	protected String name = "Acronyms";
    protected String description = "Expands acronyms defined in _acronym page. " +
        "Example: definition 'HTML=Hyper Text Markup Language' " +
        "becomes &lt;acronym title='Hyper Text Markup Language'&gt;HTML&lt;/acronym&gt;. " +
        "You must create an " +
        "<a href='page.do?method=editPages&rmik=tabbedmenu.website.pages'>" +
        "_acronym page</a> to use Acronyms.";
    
	/**
	 * Constructor
	 */
	public AcronymsPlugin()
	{
		super();
        mLogger.debug("AcronymsPlugin instantiated.");
	}

	/**
	 * @see org.apache.roller.presentation.velocity.PagePlugin#init(org.apache.roller.presentation.RollerRequest, org.apache.velocity.context.Context)
	 */
	public void init(
            WebsiteData website,
            Object config,
            String baseURL,
            Context ctx) throws RollerException
	{
		if (mLogger.isDebugEnabled()) {
			mLogger.debug("init(ctx = "+ctx+" )");
		}
	}
    
    /**
     * Look for any _acronyms Page and parse it into Properties.
     * @param website
     * @return
     * @throws RollerException
     */
    private Properties loadAcronyms(WebsiteData website) 
    {
        Properties acronyms = new Properties();
        try
        {
            UserManager userMgr = RollerFactory.getRoller().getUserManager();
            WeblogTemplate acronymsPage = userMgr.getPageByName(
                                        website, "_acronyms");
            if (acronymsPage != null) 
            {
                acronyms = parseAcronymPage(acronymsPage, acronyms);
            }
        }
        catch (RollerException e)
        {
            // not much we can do about it
            mLogger.warn(e);
        }
        return acronyms;
    }

    /**
	 * @see org.apache.roller.presentation.velocity.PagePlugin#render(org.apache.roller.pojos.WeblogEntryData, boolean)
	 */
	public String render(WeblogEntryData entry, String str)
	{
        String text = str;
        
		if (mLogger.isDebugEnabled()) {
			mLogger.debug("render(entry = "+entry.getId()+")");
		}

        /*
         * Get acronyms Properties.
         */
        Properties acronyms = loadAcronyms(entry.getWebsite());
        mLogger.debug("acronyms.size()=" + acronyms.size());
        if (acronyms.size() == 0)
        {
            return text;
        }

        /*
         * Compile the user's acronyms into RegEx patterns.
         */
        Pattern[] acronymPatterns = new Pattern[acronyms.size()];
        String[] acronymTags = new String[acronyms.size()];
        int count = 0;
        for (Iterator iter = acronyms.keySet().iterator(); iter.hasNext();)
        {
            String acronym = (String) iter.next();
            acronymPatterns[count] = Pattern.compile("\\b" + acronym + "\\b");
            mLogger.debug("match '" + acronym + "'");
            acronymTags[count] =
                "<acronym title=\""
                + acronyms.getProperty(acronym)
                + "\">"
                + acronym
                + "</acronym>";
            count++;
        }
        
        // if there are none, no work to do
        if (acronymPatterns == null || acronymPatterns.length == 0) {
            return text;
        }

        return matchAcronyms(text, acronymPatterns, acronymTags);
	}

	/**
     * Without Website cannot lookup _acronyms page.
	 * @see org.apache.roller.presentation.velocity.PagePlugin#render(java.lang.String)
	 */
	public String render(String text)
	{
	    return text;
	}

	/**
	 * @return this Page Plugin's name
	 */
	public String toString()
	{
		return name;
	}
	
	/**
	 * Iterates through the acronym properties and replaces matching 
	 * acronyms in the entry text with acronym html-tags.
	 * 
	 * @param text entry text
	 * @param acronyms user provided set of acronyms
	 * @return entry text with acronym explanations
	 */
	private String matchAcronyms(String text, Pattern[] acronymPatterns, String[] acronymTags)
	{
		if (mLogger.isDebugEnabled()) {
			mLogger.debug("matchAcronyms("+text+")");
		}

        Matcher matcher = null;
		for (int i=0; i<acronymPatterns.length; i++)
		{
            matcher = acronymPatterns[i].matcher(text);
            text = matcher.replaceAll(acronymTags[i]);
		}
		return text;
	}

	/**
	 * Parse the Template of the provided WeblogTemplate and turns it
	 * into a <code>Properties</code> collection.
	 * 
	 * @param acronymPage
	 * @return acronym properties (key = acronym, value= full text), empty if Template is empty
	 */
	private Properties parseAcronymPage(WeblogTemplate acronymPage, Properties acronyms)
	{
		String rawAcronyms = acronymPage.getContents();
		
		if (mLogger.isDebugEnabled()) 
		{
			mLogger.debug("parsing _acronyms template: \n'"+rawAcronyms+"'");
		}
		
		String regex = "\n"; // end of line
		String[] lines = rawAcronyms.split(regex);

		if (lines != null)
		{
			for (int i = 0; i < lines.length; i++)
			{
				int index = lines[i].indexOf('=');
				if (index > 0)
				{
					String key = lines[i].substring(0, index).trim();
					String value =
						lines[i].substring(index + 1, lines[i].length()).trim();
					acronyms.setProperty(key, value);
				}
			}
		}

		return acronyms;
	}

    public String getName() { return name; }
    public String getDescription() { return StringEscapeUtils.escapeJavaScript(description); }

    public boolean getSkipOnSingleEntry() {return false;}
}
